/*
	
	ChangeChannel.cpp
	
	By: Matt McMinnn (melfina@earthlink.net)
		
	A little program to change the channel on the tv tuner from 
	the command line.
	
	Version 1.0
	
*/

#include "MediaRoster.h"
#include "ParameterWeb.h"
#include "Application.h"
#include "TimeSource.h"
#include <stdio.h>

BMediaRoster *gMediaRoster;
BParameterWeb *gParameterWeb;
media_node gMediaNode;

int main(int argc, char *argv[]){

        status_t err;
		if(argc < 2){
		
			printf("Usage: ChangeChannel channel\n\n");
			printf("channel = Channel to change to.\n");
			exit(1);
		
		}
		
        new BApplication ("application/x-vnv.msm-changeChannel");
        gMediaRoster = BMediaRoster::Roster(&err);

        if( !gMediaRoster || (err != B_OK)){

                printf("No roster found - this is more difficult a problem ");
                printf("then I can handle, so I'm bailing out...\n");
                return 1;
                
        }

        err = gMediaRoster->GetVideoInput(&gMediaNode);
        err = gMediaRoster->GetParameterWebFor(gMediaNode , &gParameterWeb);
        BTimeSource* timeSource = gMediaRoster->MakeTimeSourceFor(gParameterWeb->Node());
		bigtime_t now = timeSource->Now();
		BDiscreteParameter* gParam;

		/*
		
			Checking each parameter in the web didn't catch the 
			B_TUNER_CHANNEL constant, so I explicitly declare which 
			parameter index it is - I don't know if this will always
			work, but it works for now, on my machine - this occurs
			later in the code as well.
			
		*/

		BParameter* param = gParameterWeb->ParameterAt(0);
		
		if (param->Type() == BParameter::B_DISCRETE_PARAMETER) {
	
			gParam = dynamic_cast<BDiscreteParameter*>(param);
	
		}
		else
		{
	
			printf("ERROR: expected the system mixer to be a BContinuousParameter!\n");
			exit(1);
	
		}
		int j = 0;
		int32 newChan;
		bool found = false;
		
		for(j = 0;j < gParam->CountItems();j++){
		
			if(strcmp(argv[1], gParam->ItemNameAt(j)) == 0){
			
				newChan = gParam->ItemValueAt(j);
				found = true;
				break;
				
			}
		
		}
		if(!found){
		
			printf("Channel not found.\n");
			exit(1);
			
		}
		gParam->SetValue(&newChan, sizeof(newChan), now);

		/*
		
			On occasion, changing the channel would change the 
			Audio In parameter to Mute for some reason.  This 
			just explicitly sets it back to the first choice, 
			which on my system is Tuner, which is correct.
			
		*/
		
		param = gParameterWeb->ParameterAt(2);
		
		if (param->Type() == BParameter::B_DISCRETE_PARAMETER) {
	
			gParam = dynamic_cast<BDiscreteParameter*>(param);
	
		}
		else
		{
	
			printf("Strange parameter error...\n");
			exit(1);
	
		}
		int32 aud = 0;
		gParam->SetValue(&aud, sizeof(aud), now);

        return 0;

}
  